/*
 * ntp_authprov.h - definitions for authentication providers
 */

#ifndef _NTP_AUTHPROV_H
#define _NTP_AUTHPROV_H

/*
 * An authentication provider backend, allocated by the provider's
 * constructor.
 */

struct info_auth;

#define	KEY_TYPE_MD5	4 /* type argument to setkey method */

struct authkey_be {
	void (*destructor)(struct authkey_be *be);
	int (*getmac)(struct authkey_be *be, keyid_t keyno, u_int32 *pkt, int length);
	int (*verifymac)(struct authkey_be *be, keyid_t keyno, u_int32 *pkt, int length, int size);
	void (*agekeys)(struct authkey_be *be);
	int (*havekey)(struct authkey_be *be, keyid_t keyno, int cache);
	void (*trust)(struct authkey_be *be, keyid_t keyno, u_long trust);
	int (*istrusted)(struct authkey_be *be, keyid_t keyno);
	int (*setkey)(struct authkey_be *be, keyid_t keyno, int type, const u_char *key, const int len, u_long trust);
	int (*refresh)(struct authkey_be *be);
	int (*stats)(struct authkey_be *be, struct info_auth *stats);
	void (*reset_stats)(struct authkey_be *be);
};

/*
 * Current authentication provider version
 */
#define AUTHPROV_VERSION_V1	1
#define AUTHPROV_VERSION	AUTHPROV_VERSION_V1

/*
 * Authentication provider constructor, allocates backend object
 * (provider specific state can be stored at end of structure)
 */
typedef struct authkey_be *(*authkey_be_constructor)(int version, int argc, const char *argv[]);

/*
 * Authentication provider API
 */
extern	int	authprov_register P((void *, struct authkey_be *, keyid_t[2], int));
extern	struct authkey_be *authprov_select P((keyid_t keyno));
extern	int	authprov_load	P((const char *, keyid_t[2], int , const char *[]));
extern	void	authprov_cleanup P((void));
extern	void	authprov_refresh P((void));
extern	int	authprov_stats	P((struct info_auth *ia));
extern	void	authprov_reset_stats	P((void));
extern	int	authprov_genkeyid P((void *, keyid_t *));

/*
 * Internal key file authentication provider
 */

#define KEYFILE_AUTHPROV_BE	((void *)0) /* for use with authprov_genkeyid() */

extern	struct authkey_be *keyfile_authprov_init P((int, int, const char *[]));
extern	void	keyfile_authprov_delkeys P((struct authkey_be *be));
extern	int	keyfile_authprov_readkeys P((struct authkey_be *be, const char *file));
extern	int	keyfile_authprov_setkey P((struct authkey_be *be, keyid_t keyno, int type, const u_char *key, const int len, u_long trust));

/*
 * Internal autokey authentication provider
 */
#define AUTOKEY_AUTHPROV_BE	((void *)~0) /* for use with authprov_genkeyid() */

extern	struct authkey_be *autokey_authprov_init P((int, int, const char *[]));
extern	int	autokey_iskeyid P((keyid_t));
extern	void	autokey_untrust P((keyid_t keyid));

#endif /* _NTP_AUTHPROV_H */
